@@ -1,6 +1,6 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
  | Copyright (C) 2008-2014, The Roundcube Dev Team                       |
@@ -104,21 +104,21 @@ class rcube_config
      */
     private function guess_type($value)
     {
-        $_ = 'string';
+        $type = 'string';
 
         // array requires hint to be passed.
 
         if (preg_match('/^[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?$/', $value) !== false) {
-            $_ = 'double';
+            $type = 'double';
         }
         else if (preg_match('/^\d+$/', $value) !== false) {
-            $_ = 'integer';
+            $type = 'integer';
         }
         else if (preg_match('/(t(rue)?)|(f(alse)?)/i', $value) !== false) {
-            $_ = 'boolean';
+            $type = 'boolean';
         }
 
-        return $_;
+        return $type;
     }
 
     /**
@@ -127,7 +127,7 @@ class rcube_config
      * Perform an appropriate parsing of the string to create the desired PHP type.
      *
      * @param $string String to parse into PHP type
-     * @param $type Type of value to return
+     * @param $type   Type of value to return
      *
      * @return Appropriately typed interpretation of $string.
      */
@@ -168,28 +168,24 @@ class rcube_config
      * Retrieve an environment variable's value or if it's not found, return the
      * provided default value.
      *
-     * @param $varname Environment variable name
+     * @param $varname       Environment variable name
      * @param $default_value Default value to return if necessary
-     * @param $type Type of value to return
+     * @param $type          Type of value to return
      *
      * @return Value of the environment variable or default if not found.
      */
     private function getenv_default($varname, $default_value, $type = null)
     {
-        $_ = getenv($varname);
+        $value = getenv($varname);
 
-        if ($_ === false) {
-            $_ = $default_value;
+        if ($value === false) {
+            $value = $default_value;
         }
         else {
-            if (is_null($type)) {
-                $type = gettype($default_value);
-            }
-
-            $_ = $this->parse_env($_, $type);
+            $value = $this->parse_env($value, $type ?: gettype($default_value));
         }
 
-        return $_;
+        return $value;
     }
 
     /**
@@ -207,8 +203,7 @@ class rcube_config
         // load main config file
         if (!$this->load_from_file('config.inc.php')) {
             // Old configuration files
-            if (!$this->load_from_file('main.inc.php') ||
-                !$this->load_from_file('db.inc.php')) {
+            if (!$this->load_from_file('main.inc.php') || !$this->load_from_file('db.inc.php')) {
                 $this->errors[] = 'config.inc.php was not found.';
             }
             else if (rand(1,100) == 10) {  // log warning on every 100th request (average)
@@ -230,8 +225,9 @@ class rcube_config
         }
 
         // larry is the new default skin :-)
-        if ($this->prop['skin'] == 'default')
+        if ($this->prop['skin'] == 'default') {
             $this->prop['skin'] = self::DEFAULT_SKIN;
+        }
 
         // fix paths
         foreach (array('log_dir' => 'logs', 'temp_dir' => 'temp') as $key => $dir) {
@@ -314,7 +310,7 @@ class rcube_config
 
         foreach ($this->resolve_paths($file) as $fpath) {
             if ($fpath && is_file($fpath) && is_readable($fpath)) {
-                // use output buffering, we don't need any output here 
+                // use output buffering, we don't need any output here
                 ob_start();
                 include($fpath);
                 ob_end_clean();
@@ -353,8 +349,9 @@ class rcube_config
             // check if <file>-env.ini exists
             if ($realpath && $use_env && !empty($this->env)) {
                 $envfile = preg_replace('/\.(inc.php)$/', '-' . $this->env . '.\\1', $realpath);
-                if (is_file($envfile))
+                if (is_file($envfile)) {
                     $realpath = $envfile;
+                }
             }
 
             if ($realpath) {
@@ -387,8 +384,7 @@ class rcube_config
         }
 
         $result = $this->getenv_default('ROUNDCUBE_' . strtoupper($name), $result);
-
-        $rcube = rcube::get_instance();
+        $rcube  = rcube::get_instance();
 
         if ($name == 'timezone') {
             if (empty($result) || $result == 'auto') {
@@ -396,10 +392,12 @@ class rcube_config
             }
         }
         else if ($name == 'client_mimetypes') {
-            if ($result == null && $def == null)
+            if (!$result && !$def) {
                 $result = 'text/plain,text/html,text/xml,image/jpeg,image/gif,image/png,image/bmp,image/tiff,application/x-javascript,application/pdf,application/x-shockwave-flash';
-            if ($result && is_string($result))
+            }
+            if ($result && is_string($result)) {
                 $result = explode(',', $result);
+            }
         }
 
         $plugin = $rcube->plugins->exec_hook('config_get', array(
@@ -501,12 +499,13 @@ class rcube_config
      * Return requested DES crypto key.
      *
      * @param string $key Crypto key name
+     *
      * @return string Crypto key
      */
     public function get_crypto_key($key)
     {
         // Bomb out if the requested key does not exist
-        if (!array_key_exists($key, $this->prop)) {
+        if (!array_key_exists($key, $this->prop) || empty($this->prop[$key])) {
             rcube::raise_error(array(
                 'code' => 500, 'type' => 'php',
                 'file' => __FILE__, 'line' => __LINE__,
@@ -514,18 +513,17 @@ class rcube_config
             ), true, true);
         }
 
-        $key = $this->prop[$key];
+        return $this->prop[$key];
+    }
 
-        // Bomb out if the configured key is not exactly 24 bytes long
-        if (strlen($key) != 24) {
-            rcube::raise_error(array(
-                'code' => 500, 'type' => 'php',
-                'file' => __FILE__, 'line' => __LINE__,
-                'message' => "Configured crypto key '$key' is not exactly 24 bytes long"
-            ), true, true);
-        }
-
-        return $key;
+    /**
+     * Return configured crypto method.
+     *
+     * @return string Crypto method
+     */
+    public function get_crypto_method()
+    {
+        return $this->get('cipher_method') ?: 'DES-EDE3-CBC';
     }
 
     /**
@@ -573,8 +571,9 @@ class rcube_config
         $domain = $host;
 
         if (is_array($this->prop['mail_domain'])) {
-            if (isset($this->prop['mail_domain'][$host]))
+            if (isset($this->prop['mail_domain'][$host])) {
                 $domain = $this->prop['mail_domain'][$host];
+            }
         }
         else if (!empty($this->prop['mail_domain'])) {
             $domain = rcube_utils::parse_host($this->prop['mail_domain']);
